---
title: Publish an ASP.NET Core app to IIS
author: guardrex
description: Learn how to host an ASP.NET Core app on an IIS server.
monikerRange: '>= aspnetcore-2.1'
ms.author: riande
ms.custom: mvc
ms.date: 08/08/2019
uid: tutorials/publish-to-iis
---
# Publish an ASP.NET Core app to IIS

By [Luke Latham](https://github.com/guardrex)

This tutorial shows how to host an ASP.NET Core app on an IIS server.

This tutorial covers the following subjects:

> [!div class="checklist"]
> * Install the .NET Core Hosting Bundle on Windows Server.
> * Create an IIS site in IIS Manager.
> * Deploy an ASP.NET Core app.

## Prerequisites

* [.NET Core SDK](/dotnet/core/sdk) installed on the development machine.
* Windows Server configured with the **Web Server (IIS)** server role. If your server isn't configured to host websites with IIS, follow the guidance in the *IIS configuration* section of the <xref:host-and-deploy/iis/index#iis-configuration> article and then return to this tutorial.

> [!WARNING]
> **IIS configuration and website security involve concepts that aren't covered by this tutorial.** Consult the IIS guidance in the [Microsoft IIS documentation](https://www.iis.net/) and the [ASP.NET Core article on hosting with IIS](xref:host-and-deploy/iis/index) before hosting production apps on IIS.
>
> Important scenarios for IIS hosting not covered by this tutorial include:
>
> * [Creation of a registry hive for ASP.NET Core Data Protection](xref:host-and-deploy/iis/index#data-protection)
> * [Configuration of the app pool's Access Control List (ACL)](xref:host-and-deploy/iis/index#application-pool-identity)
> * To focus on IIS deployment concepts, this tutorial deploys an app without HTTPS security configured in IIS. For more information on hosting an app enabled for HTTPS protocol, see the security topics in the [Additional resources](#additional-resources) section of this article. Further guidance for hosting ASP.NET Core apps is provided in the <xref:host-and-deploy/iis/index> article.

## Install the .NET Core Hosting Bundle

Install the *.NET Core Hosting Bundle* on the IIS server. The bundle installs the .NET Core Runtime, .NET Core Library, and the [ASP.NET Core Module](xref:host-and-deploy/aspnet-core-module). The module allows ASP.NET Core apps to run behind IIS. If the system doesn't have an Internet connection, obtain and install the [Microsoft Visual C++ 2015 Redistributable](https://www.microsoft.com/download/details.aspx?id=53840) before installing the .NET Core Hosting Bundle.

Download the installer using the following link:

[Current .NET Core Hosting Bundle installer (direct download)](https://www.microsoft.com/net/permalink/dotnetcore-current-windows-runtime-bundle-installer)

1. Run the installer on the IIS server.

1. Restart the server or execute **net stop was /y** followed by **net start w3svc** in a command shell.

## Create the IIS site

1. On the IIS server, create a folder to contain the app's published folders and files. In a following step, the folder's path is provided to IIS as the physical path to the app.

1. In IIS Manager, open the server's node in the **Connections** panel. Right-click the **Sites** folder. Select **Add Website** from the contextual menu.

1. Provide a **Site name** and set the **Physical path** to the app's deployment folder that you created. Provide the **Binding** configuration and create the website by selecting **OK**.

## Create an ASP.NET Core Razor Pages app

Follow the <xref:getting-started> tutorial to create a Razor Pages app.

## Publish and deploy the app

*Publish an app* means to produce a compiled app that can be hosted by a server. *Deploy an app* means to move the published app to a hosting system. The publish step is handled by the [.NET Core SDK](/dotnet/core/sdk), while the deployment step can be handled by a variety of approaches. This tutorial adopts the *folder* deployment approach, where:

* The app is published to a folder.
* The folder's contents are moved to the IIS site's folder (the **Physical path** to the site in IIS Manager).

# [Visual Studio](#tab/visual-studio)

1. Right-click on the project in **Solution Explorer** and select **Publish**.
1. In the **Pick a publish target** dialog, select the **Folder** publish option.
1. Set the **Folder or File Share** path.
   * If you created a folder for the IIS site that's available on the development machine as a network share, provide the path to the share. The current user must have write access to publish to the share.
   * If you're unable to deploy directly to the IIS site folder on the IIS server, publish to a folder on removeable media and physically move the published app to the IIS site folder on the server, which is the site's **Physical path** in IIS Manager. Move the contents of the *bin/Release/{TARGET FRAMEWORK}/publish* folder to the IIS site folder on the server, which is the site's **Physical path** in IIS Manager.

# [.NET Core CLI](#tab/netcore-cli)

1. In a command shell, publish the app in Release configuration with the [dotnet publish](/dotnet/core/tools/dotnet-publish) command:

   ```dotnetcli
   dotnet publish --configuration Release
   ```

1. Move the contents of the *bin/Release/{TARGET FRAMEWORK}/publish* folder to the IIS site folder on the server, which is the site's **Physical path** in IIS Manager.

# [Visual Studio for Mac](#tab/visual-studio-mac)

1. Right-click on the project in **Solution** and select **Publish** > **Publish to Folder**.
1. Set the **Choose a folder** path.
   * If you created a folder for the IIS site that's available on the development machine as a network share, provide the path to the share. The current user must have write access to publish to the share.
   * If you're unable to deploy directly to the IIS site folder on the IIS server, publish to a folder on removeable media and physically move the published app to the IIS site folder on the server, which is the site's **Physical path** in IIS Manager. Move the contents of the *bin/Release/{TARGET FRAMEWORK}/publish* folder to the IIS site folder on the server, which is the site's **Physical path** in IIS Manager.

---

## Browse the website

The app is accessible in a browser after it receives the first request. Make a request to the app at the endpoint binding that you established in IIS Manager for the site.

## Next steps

In this tutorial, you learned how to:

> [!div class="checklist"]
> * Install the .NET Core Hosting Bundle on Windows Server.
> * Create an IIS site in IIS Manager.
> * Deploy an ASP.NET Core app.

To learn more about hosting ASP.NET Core apps on IIS, see the IIS Overview article:

> [!div class="nextstepaction"]
> <xref:host-and-deploy/iis/index>

## Additional resources

### Articles in the ASP.NET Core documentation set

* <xref:host-and-deploy/aspnet-core-module>
* <xref:host-and-deploy/directory-structure>
* <xref:test/troubleshoot-azure-iis>
* <xref:security/enforcing-ssl>

### Articles pertaining to ASP.NET Core app deployment

* <xref:tutorials/publish-to-azure-webapp-using-vs>
* <xref:tutorials/publish-to-azure-webapp-using-vscode>
* <xref:host-and-deploy/visual-studio-publish-profiles>
* [Publish a Web app to a folder using Visual Studio for Mac](/visualstudio/mac/publish-folder)

### Articles on IIS HTTPS configuration

* [Configuring SSL in IIS Manager](/iis/manage/configuring-security/configuring-ssl-in-iis-manager)
* [How to Set Up SSL on IIS](/iis/manage/configuring-security/how-to-set-up-ssl-on-iis)

### Articles on IIS and Windows Server

* [The Official Microsoft IIS Site](https://www.iis.net/)
* [Windows Server technical content library](/windows-server/windows-server)
